@@ -76,7 +76,7 @@ struct OutOfVectorSize : std::exception{
   OutOfVectorSize() { }
   virtual ~OutOfVectorSize() throw() {}
   virtual const char* what() const noexcept {
-      static const std::string msg = "Out of vector size!";
+    static const std::string msg = "Out of vector size!";
       return msg.c_str();
   }
 };
@@ -127,6 +127,15 @@ struct WrongType : public InvalidInput{
 };
 
 
+struct InvalidSampleInVcf : public InvalidInput{
+  explicit InvalidSampleInVcf(string sample, string vcf):InvalidInput(sample) {
+    this->reason = "Invalid sample name: ";
+    throwMsg = this->reason + this->src + " in " + vcf;
+  }
+  ~InvalidSampleInVcf() throw() {}
+};
+
+
 struct BadScientificNotation : public InvalidInput{
   explicit BadScientificNotation(string str1, string str2):InvalidInput(str1) {
     this->reason = "Bad scientific notation: ";
@@ -183,6 +192,18 @@ struct FlagsConflict : public InvalidInput{
 };
 
 
+struct FlagsOrderIncorrect : public InvalidInput{
+  // -plafFromVcf after -vcf
+  // -sample after -vcf
+  explicit FlagsOrderIncorrect(string str1, string str2):InvalidInput(str1) {
+    this->reason = "Flag: ";
+    throwMsg = this->reason + this->src +
+      string(" should be used after flag ") + str2;
+  }
+  ~FlagsOrderIncorrect() throw() {}
+};
+
+
 struct OutOfRange : public InvalidInput{
   explicit OutOfRange(string str1, string str2):InvalidInput(str1) {
     this->reason = "Flag \"";
