@@ -29,6 +29,7 @@
 #include <iostream>
 #include <cmath>
 #include <vector>
+#include <cstdint>
 
 
 // Number of interpolation points.  If this is changed, several constants in fastlog must also be changed.
@@ -85,8 +86,8 @@ inline double FastFunc::fastexp_lo(double y) {
 
 inline double FastFunc::fastlog(double x) {
   const float offset = 2047;                // as int64_t: 0x409ffc00000....
-  double y = x;
-  int64_t* yint = (int64_t*)(&y);
+  //double y = x;   int64_t* yint = reinterpret_cast<int64_t*>(&y);
+  int64_t* yint = reinterpret_cast<int64_t*>(&x);
   int expon = ((*yint) >> 52) - 1023;       // base-2 exponent of float
   int index = ((*yint) >> (52-10)) & 1023;  // upper 10 bits of mantissa
   *yint |= 0x7ffffc0000000000;              // convert float into remainder of mantissa; and
